% Example Program to Measure sVSWR 
% The following example program performs automatically a TD sVSWR 
% measurement with a vector network analyzer. The example program is 
% written for MATLAB.  It requires a VNA with time domain option.
%
% use Keysight PNA series VNA to measure time domain sVSWR
%

% Before performing the test, user need to manually set:
%  1. To measure S21
%  2. To perform cable calibrations 
%  3. To set parameters manually including IFBW, start/stop frequency,
%  4. To set number of points, gate shape, windowing function and source power. 


%%% User defined parameters %%%
PNAaddress=16;
GATESPAN=4.5e-9; %time gate span in s
%specify frequency range of interest (for removing bandedge data).  
WFreqStart=1e9;
WFreqEnd=10e9;

%specify coverage factor for the statistical analysis
kfactor=0.676;
%%%%
% 
dnominal=3; %nominal distance, typically 3m, not needed in TD (used in CISPR)
%quietzone radius
radius=0.5; %not needed in TD


%closes and deletes all instruments first
ins=instrfind;
NumberOfInstruments = length(ins);
for i=1:NumberOfInstruments
	fclose(ins(i))
	delete(ins(i))
end

%if(~exist('vna','var'))
vna=gpib('ni',0,PNAaddress); 
%vna=visa('NI','GPIB0::16::0::INSTR'); %use this if gpib doesnot work 
%vna=visa('Agilent','TCPIP0::LOCALHOST::INST0::INSTR'); % if call directly in PNA
vna.InputBufferSize = 600000;
%vna.Timeout=30;
fopen(vna);
%end
%
clrdevice(vna);
fprintf(vna,'*IDN?');
out=fscanf(vna)  %

while(1)  %get test location L1...L6/R1...R6/C1...C6/F1...F6
    testlocation=input('input test location (must start with F1...F6,C1...C6 etc) =','s');
    testlocation=upper(testlocation);
    switch(testlocation(1))
        case{'L','l','R','r'}
        dref=sqrt((dnominal+radius)^2+radius^2)-0.4;
        if any(str2num(testlocation(2))==[1:6])
            break;
        end
        case{'C','c'}
        dref=dnominal+radius;
        if any(str2num(testlocation(2))==[1:6])
            break;
        end
        case{'f','F'}
        dref=dnominal;
        if any(str2num(testlocation(2))==[1:6])
            break;
        end
        case{'B','b'}
        dref=dnominal+2*radius;
        if any(str2num(testlocation(2))==[1:6])
            break;
        end
        otherwise
        display('Not a valid input');
        end;
end

fprintf(vna,'SENSe1:SWEep:POIN?'); % number of points
numpts=fscanf(vna,'%g');
fprintf(vna,'SENS:FREQ:START?');
freqstart=fscanf(vna,'%g');
fprintf(vna,'SENS:FREQ:STOP?');
freqend=fscanf(vna,'%g');
freqs=freqstart+([1:numpts]-1).*(freqend-freqstart)./(numpts-1);  %all freqs

% Set byte order to swapped (little-endian) format
fprintf(vna, 'FORM:BORD SWAP');
% Set data type to real 64 bit binary block
fprintf(vna, 'FORM REAL,64');


%"CALC:PAR:MNUM:SEL 1" may not work in older PNAs 
% use "CALC:PAR:CAT?" to list the names
% and then for example use "CALC:PAR:SEL 'CH1_S11_1'" to select
% CHchNum_[measurement name]_{trace number}
% Note when [measurement name] is S11, it doesn't mean you are measuring s11.  This
% is the name created by the system when the measurement is first setup
% "SYST:ACT:CHAN?" returns the active channel number
% "SYST:ACT:MEAS?" returns the active measurement name.
% "CALC:PAR:TNUM?" returns the trace number of the selected trace

fprintf(vna,'CALC:PAR:MNUM:SEL 1')

fprintf(vna,'CALC:FORM MLOG'); %log magnitude
fprintf(vna,'CALC:FILTer:GATE:TIME:STATE OFF'); %turns off time gating

%trigger a measurement and wait for it to finish
%fprintf(vna,'ABORT; :INITIATE:IMMEDIATE');  %restart the measurement
fprintf(vna,'ABORT');
%fprintf(vna,'*WAI');
display('wait for sweep to finish.  Press key to continue...');
pause;

% Non-gated data

%fprintf(vna,'CALC:FORM MLOG'); %log magnitude
%fprintf(vna,'CALC:FILTer:GATE:TIME:STATE OFF'); %turns off time gating

fprintf(vna,'CALCulate:DATA? FDATA'); % get data
%FDATA - Formatted Meas, after gating
%FMEM - Formatted memory, after gating
%SDATA - Corrected, Complex Meas before gating
%SMEM - Corrected, Complex Memory before gating
[mag_gnot, count, msg] = binblockread(vna, 'double');
clrdevice(vna);

fprintf(vna,'CALC:FORM PHASE');
fprintf(vna,'CALC:DATA? FDATA');
[phase_gnot,count,msg]= binblockread(vna, 'double');
clrdevice(vna);

%time domain gating
fprintf(vna,'CALC:FORM MLOG'); %log magnitude
fprintf(vna,'CALC:TRANSFORM:TIME:STATE ON'); % turns on time domain
fprintf(vna,'CALC:TRANSFORM:TIME:START 0e-9'); %start/stop time
fprintf(vna,'CALC:TRANSFORM:TIME:STOP 200e-9');

fprintf(vna,'CALC:MARKER1:STATE ON'); %turns on marker
fprintf(vna,'CALC:MARK:FUNC:TRAC ON'); % turns on tracking so max is read for every sweep
fprintf(vna,'CALC:MARKER1:FUNC:SELECT MAX'); %find max for marker
fprintf(vna,'CALC:MARK:X?'); %read marker position
Tpeak=str2num(fscanf(vna));  %this is peak of the main response

fprintf(vna,['CALC:FILTER:GATE:TIME:CENTER ',num2str(Tpeak)]); %set gate center at main response
fprintf(vna,['CALC:FILTER:GATE:TIME:SPAN ',num2str(GATESPAN)]); %time gate span

fprintf(vna,'CALC:FILTER:GATE:TIME:TYPE BPASS'); %set bandpass filter,i.e. Gate-in

fprintf(vna,'CALC:FILTer:GATE:TIME:STATE ON'); %turns on time gating
fprintf(vna,'CALC:TRANSFORM:TIME:STATE OFF'); % return to freq domain

% gate_in data
fprintf(vna,'CALC:FORM MLOG'); %log magnitude
fprintf(vna,'CALC:DATA? FDATA');
[mag_gin,count,msg]= binblockread(vna, 'double');

fprintf(vna,'CALC:FORM PHASE');
fprintf(vna,'CALC:DATA? FDATA');
[phase_gin,count,msg]= binblockread(vna, 'double');

% gate_out data
fprintf(vna,'CALC:FILTER:GATE:TIME:TYPE NOTCH'); %set NOTCH filter, Gate-out
fprintf(vna,'CALC:FORM MLOG'); %log magnitude
fprintf(vna,'CALC:DATA? FDATA');
[mag_gout,count,msg]= binblockread(vna, 'double');

fprintf(vna,'CALC:FORM PHASE');
fprintf(vna,'CALC:DATA? FDATA');
[phase_gout,count,msg]= binblockread(vna, 'double');

%return to log mag screen, and no time gating
fprintf(vna,'CALC:FORM MLOG'); %log magnitude
fprintf(vna,'CALC:FILTer:GATE:TIME:STATE OFF'); %turns off time gating

%post process, calculating sVSWR
freqTD=freqs;
rhodB=mag_gout-mag_gin;
rho=10.^(rhodB/20);
TDvswrdB=abs(20*log10((1+rho)./(1-rho)));


%-------------------------- Moving average/std
freqWindowHz=120e6;  %set moving avg window, e.g., approx 120 MHz
windowsize=freqWindowHz/(freqTD(2)-freqTD(1))+1;
windowsize=2.*round((windowsize+1)/2)-1; %round to nearest odd number
display(['Number of points in the moving window = ',num2str(windowsize)]);

TDwindowsize=windowsize;  % the window sizes need to be adjusted based on autocorrelation results!
  TDmavg=movingmean(TDvswrdB,TDwindowsize,[],[]);
  TDstd1=movingstd(TDvswrdB-TDmavg,floor(TDwindowsize/2),'central');
%        movingstd:  windowmode = 'central' --> use a sliding window centered on
%            each point in the series. The window will have total width
%            of 2*k+1 points, thus k points on each side.
display(['TD moving window (Hz) = ',num2str(freqTD(TDwindowsize)-freqTD(1))]);

%------------------------------- plotting all data
figure(1),
plot(freqTD,TDvswrdB,freqTD,(TDmavg+kfactor*TDstd1)',freqTD,6*ones(1,length(freqTD)),'r','LineWidth',2);
axis([WFreqStart WFreqEnd 0 10])
title(['sVSWR by TD, position=',testlocation]);
xlabel('frequency (Hz)');
ylabel('sVSWR (dB)');
legend('TD all',['TDmavg+',num2str(kfactor),'\sigma']);
grid;

fclose(vna);
delete(vna);
clear vna;

dlmwrite(['TD_',testlocation,'.csv'],[freqs',TDvswrdB,TDmavg,TDstd1,TDmavg+kfactor*TDstd1]);
