% Example Program to Measure sVSWR 
% The following example program performs automatically a TD sVSWR 
% measurement with a vector network analyzer. The example program is 
% written for MATLAB.  It requires a VNA with time domain option.
%
% use Rohde&Schwarz ZVA/ZNB series VNA to measure time domain sVSWR
%

% Before performing the test, user need to manually set:
%  1. To measure S21
%  2. To perform cable calibrations 
%  3. To set parameters manually including IFBW, start/stop frequency,
%  4. To set number of points, gate shape, windowing function and source power. 

%%% User defined parameters %%%
ZVAaddress=20;
ZVAip='169.254.1.30'
GATESPAN=4.5e-9; %time gate span in s, should be double of antenna ringdown

%specify coverage factor for the statistical analysis
kfactor=0.676;

%closes and deletes all instruments first
ins=instrfind;
NumberOfInstruments = length(ins);
for i=1:NumberOfInstruments
  fclose(ins(i))
  delete(ins(i))
end

% List of different choices to connect to the VNA.
vna = tcpip(ZVAip,5025);
%vna = rstcpip(ZVAip,5025);
%vna=gpib('ni',0,ZVAaddress); 
%vna=visa('NI','GPIB0::20::0::INSTR'); %for VISA 
vna.InputBufferSize = 600000;
fopen(vna);
%
fprintf(vna,'*CLS');
fprintf(vna,':SYST:DISP:UPD ON');
fprintf(vna,'*IDN?');
out=fscanf(vna)

while(1)  %get test location L1...L6/R1...R6/C1...C6/F1...F6
    testlocation=input('input test location (must start with F1...F6,C1...C6 etc) = ','s');
    testlocation=upper(testlocation);
    if any(str2num(testlocation(2))==[1:6])
        break;
    else
        display('Not a valid input');
    end
end
fprintf(vna,'SENSe1:SWEep:POIN?'); % read number of points
numpts=fscanf(vna,'%g');
fprintf(vna,'SENS:FREQ:START?');  %read start frequency
freqstart=fscanf(vna,'%g');
fprintf(vna,'SENS:FREQ:STOP?'); %read stop frequency
freqend=fscanf(vna,'%g');
freqs=freqstart+([1:numpts]-1).*(freqend-freqstart)./(numpts-1);  %all freqs

% Set byte order to normal (big-endian) format
fprintf(vna, 'FORM:BORD NORM');
% Set data type to real 64 bit binary block
fprintf(vna, 'FORM REAL,64');

fprintf(vna,'CALC:TRANSFORM:TIME:STATE OFF'); % turns on time domain
fprintf(vna,'CALC:PAR:SEL "Trc1"'); % select trace


fprintf(vna,'CALC:FORM MLOG'); %log magnitude
fprintf(vna,'CALC:FILTer:GATE:TIME:STATE OFF'); %turns off time gating

%trigger a meafsurement and wait for it to finish
fprintf(vna,':INITIATE:CONT OFF');  %set instrument to single sweep
fprintf(vna,':INITIATE:IMMEDIATE;*OPC?');
out=fscanf(vna);

% Non-gated data

fprintf(vna,'CALCulate:DATA? FDATA'); % get data
%FDATA - Formatted Meas, after gating
[mag_gnot, count, msg] = binblockread(vna, 'double');

fprintf(vna,'CALC:FORM PHASE');
fprintf(vna,'CALC:DATA? FDATA');
[phase_gnot,count,msg]= binblockread(vna, 'double');

% with time domain gating
fprintf(vna,'CALC:FORM MLOG'); %log magnitude
fprintf(vna,'CALC:TRANSFORM:TIME:STATE ON'); % turns on time domain
fprintf(vna,'CALC:TRANSFORM:TIME:START 0e-9'); %start/stop time
fprintf(vna,'CALC:TRANSFORM:TIME:STOP 200e-9');
fprintf(vna,'*CLS');

fprintf(vna,':INITIATE:CONT ON');  %set instrument to continuous sweep
fprintf(vna,'CALC:MARKER1:STATE ON'); %turns on marker
fprintf(vna,'CALC:MARKER1:FUNC:EXEC MAX'); %find max for marker
fprintf(vna,'CALC:MARK:SEARCH:TRAC ON'); % turns on tracking so max is read for every sweep

% initiate sweep
fprintf(vna,'*CLS');
fprintf(vna,':INITIATE:CONT OFF');  %set instrument to single sweep
fprintf(vna,':INITIATE:IMMEDIATE;*OPC?');
out=fscanf(vna)

fprintf(vna,'CALC:MARK1:X?'); %read marker position
Tpeak=str2num(fscanf(vna));  %this is peak of the main response

fprintf(vna,['CALC:FILTER:GATE:TIME:CENTER ',num2str(Tpeak)]); %set gate center at main response
fprintf(vna,['CALC:FILTER:GATE:TIME:SPAN ',num2str(GATESPAN)]); %time gate span

fprintf(vna,'CALC:FILTER:GATE:TIME:TYPE BPASS'); %set bandpass filter,i.e. Gate-in
fprintf(vna,'CALC:FILTer:GATE:TIME:STATE ON'); %turns on time gating
fprintf(vna,'CALC:TRANSFORM:TIME:STATE OFF'); % return to freq domain
% initiate sweep
fprintf(vna,'*CLS');
fprintf(vna,':INITIATE:CONT OFF');  %set instrument to single sweep
fprintf(vna,':INITIATE:IMMEDIATE;*OPC?');
out=fscanf(vna)

% gate_in data (bandpass gate)
fprintf(vna,'CALC:FORM MLOG'); %log magnitude
fprintf(vna,'CALC:DATA? FDATA');
[mag_gin,count,msg]= binblockread(vna, 'double');

% gate_out (null gate) data
fprintf(vna,'CALC:FILTER:GATE:TIME:TYPE NOTCH'); %set NOTCH filter
fprintf(vna,'CALC:FORM MLOG'); %log magnitude
% initiate sweep
fprintf(vna,'*CLS');
fprintf(vna,':INITIATE:CONT OFF');  %set instrument to single sweep
fprintf(vna,':INITIATE:IMMEDIATE;*OPC?');
out=fscanf(vna);

fprintf(vna,'CALC:DATA? FDATA');
[mag_gout,count,msg]= binblockread(vna, 'double');


%return to log mag screen, and no time gating
fprintf(vna,'CALC:FORM MLOG'); %log magnitude
fprintf(vna,'CALC:FILTer:GATE:TIME:STATE OFF'); %turns off time gating
fprintf(vna,':INITIATE:CONT ON'); %continuous sweep

%post process, calculating sVSWR
freqTD=freqs;
rhodB=mag_gout-mag_gin;
rho=10.^(rhodB/20);
TDvswrdB=abs(20*log10((1+rho)./(1-rho)));


%-------------------------- Moving average/std
freqWindowHz=120e6;  %set moving avg window, e.g., approx 120 MHz
windowsize=freqWindowHz/(freqTD(2)-freqTD(1))+1;
windowsize=2.*round((windowsize+1)/2)-1; %round to nearest odd number
display(['Number of points in the moving window = ',num2str(windowsize)]);

TDwindowsize=windowsize;  % based on autocorrelation results
TDmavg=movingmean(TDvswrdB,TDwindowsize,[],[]);
TDstd=movingstd(TDvswrdB-TDmavg,floor(TDwindowsize/2),'central');
%        movingstd:  windowmode = 'central' --> use a sliding window centered on
%            each point in the series. The window will have total width
%            of 2*k+1 points, thus k points on each side.
display(['TD moving window (Hz) = ',num2str(freqTD(TDwindowsize)-freqTD(1))]);

%------------------------------- plotting all data
figure(1),
plot(freqTD,TDvswrdB,freqTD,(TDmavg+kfactor*TDstd)',freqTD,6*ones(1,length(freqTD)),'r','LineWidth',2);
% Uncomment this line if you want fixed scale for th Y axis
% axis([freqstart freqend 0 10])
title(['sVSWR by TD, position=',testlocation]);
xlabel('frequency (Hz)');
ylabel('sVSWR (dB)');
legend('TD all',['TDmavg+',num2str(kfactor),'\sigma']);
grid;
axis([1e9 10e9 1 10])

fclose(vna);
delete(vna);
clear vna;

dlmwrite(['TD_',testlocation,'.csv'],[freqs',TDvswrdB,TDmavg,TDstd1,TDmavg+kfactor*TDstd]);
